'''title'''
__title__ = 'pikachuwechat'
'''description'''
__description__ = 'PikachuWeChat: Play WeChat with itchat-uos.'
'''url'''
__url__ = 'https://github.com/CharlesPikachu/pikachuwechat'
'''version'''
__version__ = '0.1.1'
'''author'''
__author__ = 'Zhenchao Jin'
'''email'''
__email__ = 'charlesblwx@gmail.com'
'''license'''
__license__ = 'MIT License'
'''copyright'''
__copyright__ = 'Copyright 2021 Zhenchao Jin'